@@ -20,9 +20,7 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <libgupnp/gupnp-control-point.h>
 #include <libgupnp-av/gupnp-av.h>
@@ -146,15 +144,11 @@ on_context_available (GUPnPContextManager *cm,
 static gboolean
 init_upnp (int port)
 {
-        GUPnPWhiteList *white_list;
+        GUPnPContextFilter *context_filter;
         GtkButton *button;
         GUPnPResourceFactory *factory;
         GSocketFamily family = G_SOCKET_FAMILY_INVALID;
 
-#if !GLIB_CHECK_VERSION(2, 35, 0)
-        g_type_init ();
-#endif
-
         factory = gupnp_resource_factory_get_default ();
 
         /* Work-around bgo#764498 */
@@ -194,10 +188,10 @@ init_upnp (int port)
         g_assert (context_manager != NULL);
 
         if (interfaces != NULL) {
-                white_list = gupnp_context_manager_get_white_list
-                                            (context_manager);
-                gupnp_white_list_add_entryv (white_list, interfaces);
-                gupnp_white_list_set_enabled (white_list, TRUE);
+                context_filter = gupnp_context_manager_get_context_filter (
+                        context_manager);
+                gupnp_context_filter_add_entryv (context_filter, interfaces);
+                gupnp_context_filter_set_enabled (context_filter, TRUE);
         }
 
         g_signal_connect (context_manager,
