@@ -18,11 +18,10 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
+
+#include <stdlib.h>
+#include <string.h>
 
 #include <gmodule.h>
 
