@@ -18,12 +18,11 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <libgupnp/gupnp.h>
-#include <string.h>
-#include <stdlib.h>
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
+
+#include <libgupnp/gupnp.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include "control_point.h"
 #include "item-creation.h"
@@ -134,13 +133,9 @@ gint
 main (gint   argc,
       gchar *argv[])
 {
-        GError *error = NULL;
+        g_autoptr (GError) error = NULL;
         gint i;
-        GOptionContext *context;
-
-#if !GLIB_CHECK_VERSION(2, 35, 0)
-        g_type_init ();
-#endif
+        g_autoptr (GOptionContext) context;
 
         context = g_option_context_new ("- Upload files to UPnP MediaServer");
         g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
@@ -151,11 +146,10 @@ main (gint   argc,
         }
 
         if (argc < 2) {
-                char *help = NULL;
+                g_autofree char *help = NULL;
 
                 help = g_option_context_get_help (context, TRUE, NULL);
                 g_print ("%s\n", help);
-                g_free (help);
 
                 return -4;
         }
@@ -181,11 +175,14 @@ main (gint   argc,
         }
 
         error = NULL;
-        upnp_context = gupnp_context_new (interface, 0, &error);
+        upnp_context = gupnp_context_new_full (interface,
+                                               NULL,
+                                               0,
+                                               GSSDP_UDA_VERSION_1_0,
+                                               &error);
         if (error) {
                 g_printerr ("Error creating the GUPnP context: %s\n",
-			    error->message);
-                g_error_free (error);
+                            error->message);
 
                 return -6;
         }
@@ -206,7 +203,6 @@ main (gint   argc,
         g_clear_pointer (&main_loop, g_main_loop_unref);
         deinit_control_point ();
         g_object_unref (upnp_context);
-        g_option_context_free (context);
         g_free (dest_container);
         g_free (interface);
 
